(**********************************************************************)
(*************     bloc  inclure en mode interactif      *************)
(**********************************************************************)
(*
#directory "c:\\ocaml\\lib\\mlgraph.lib";;
#directory "c:\\ocaml\\lib\\mlgraph.lib\\mlg";;
#load "compatibility.cmo";;
#load  "prelude.cmo";;
#load  "geometry.cmo";;
#load  "paint.cmo";;
#load  "frames.cmo";;
#load  "sketches.cmo";;
#load  "bitmaps.cmo";;
#load  "fonts.cmo";;
#load  "texts.cmo";;
#load  "graphics_defaults.cmo";;
#load  "pictures.cmo";;
#load  "circletext.cmo";;
#load  "cps.cmo";;
#load  "trps.cmo";;
#load  "display.cmo";;
*)

(**********************************************************************)
(*************                   MlGraph                  *************)
(**********************************************************************)

open   Compatibility;;
open   Prelude;;
open   Geometry;;
open   Paint;;
open   Frames;;
open   Sketches;;
open   Bitmaps;;
open   Fonts;;
open   Texts;;
open   Graphics_defaults;;
open   Pictures;;
open   Circletext;;
open   Cps;;
open   Trps;;
open   Display;;

(* BEGIN COMPATIBILITY *)
(*
val lt_int : 'a -> 'a -> bool
val lt_float : 'a -> 'a -> bool
val le_float : 'a -> 'a -> bool
val gt_int : 'a -> 'a -> bool
val gt_float : 'a -> 'a -> bool
val mult_float : float -> float -> float
val add_int : int -> int -> int
val int_of_char : char -> int
val char_of_int : int -> char
val space_char : char
val lf_char : char
val char_0 : char
val comma_char : char
val open_par_char : char
val close_par_char : char
val ascii_0 : int
val ascii_9 : int
val ascii_a : int
val ascii_f : int
val ascii_A : int
val ascii_F : int
val eq_string : string -> string -> bool
val string_length : string -> int
val sub_string : string -> int -> int -> string
*)
let (map : ('a -> 'b) -> 'a list -> 'b list) = map;;
(*
val nth_char : string -> int -> char
val set_nth_char : string -> int -> char -> unit
val create_string : int -> string
val make_string : int -> char -> string
val it_list : ('a -> 'b -> 'a) -> 'a -> 'b list -> 'a
val list_it : ('a -> 'b -> 'b) -> 'a list -> 'b -> 'b
val append : 'a list -> 'a list -> 'a list
val list_length : 'a list -> int
val int_of_float : float -> int
val float_of_int : int -> float
val rev : 'a list -> 'a list
val do_list : ('a -> 'b) -> 'a list -> unit
*)

type 'a vect = 'a Compatibility.vect;;

(*
val mem : 'a -> 'a list -> bool
val vect_of_list : 'a list -> 'a array
val make_vect : int -> 'a -> 'a array
val except : 'a list -> 'a -> 'a list
val subtract : 'a list -> 'a list -> 'a list
*)
let  (assoc : 'a -> ('a * 'b) list -> 'b) = assoc;;
(*
val index : 'a -> 'a list -> int
val replace_string : string -> string -> int -> unit
val format_float : ('a, unit, string) format -> 'a
val std_out : out_channel
val assq : 'a -> ('a * 'b) list -> 'b
val combine : 'a list -> 'b list -> ('a * 'b) list
val hd : 'a list -> 'a
val tl : 'a list -> 'a list
val vect_length : 'a array -> int
*)
(* END COMPATIBILITY *)

(* BEGIN PRELUDE *)
let ( sinus) =sinus ;;
let ( cosinus) =cosinus ;;

let ( change_graphics_directory) =change_graphics_directory ;;
let ( directory_concat_string) =directory_concat_string ;;
let ( modify_body_prelude) = modify_body_prelude ;;


(* END PRELUDE *)


(* BEGIN GEOMETRY  *)

type point =  Geometry.point =
  { xc : float
  ; yc : float
  }
;;

type geom_element = Geometry.geom_element =
    Seg of point list
  | Arc of point * float * float * float
  | Curve of point * point * point * point
;;

type transformation = Geometry.transformation =
  { m11 : float
  ; m12 : float
  ; m13 : float
  ; m21 : float
  ; m22 : float
  ; m23 : float
  }
;;


let ( make_point : float * float -> point ) = make_point ;;
let ( origin:point ) = origin ;;
let ( make_transformation
 : float * float * float * float * float * float -> transformation 
) = make_transformation ;;
let ( id_trans : transformation ) = id_trans ;;
let ( transform_point : transformation -> point -> point
) = transform_point ;;
let ( compose_transformation
 : transformation -> transformation -> transformation
) = compose_transformation ;;
let ( compose_transformations : transformation list -> transformation
) = compose_transformations ;;
let ( ctrans : transformation -> transformation -> transformation
) = ctrans ;;
let ( inverse_transformation : transformation -> transformation
) = inverse_transformation ;;
let ( handle_transform : point * point -> point * point -> transformation
) = handle_transform ;;
let ( translation : float * float -> transformation ) = translation ;;
let ( origin_rotation : float -> transformation ) = origin_rotation ;;
let ( rotation : point -> float -> transformation ) = rotation ;;
let ( scaling : float * float -> transformation ) = scaling ;;
let ( symmetry : float * float -> transformation ) = symmetry ;;
let ( vsymmetry : float -> transformation ) = vsymmetry ;;
let ( hsymmetry : float -> transformation ) = hsymmetry ;;
let ( line_symmetry : point * point -> transformation ) = line_symmetry ;;
let ( point_symmetry : point -> transformation ) = point_symmetry ;;

(* END GEOMETRY *)


(* BEGIN PAINT *)


type color = Paint.color
           = Rgb of float * float * float
           | Hsb of float * float * float
           | Gra of float;;

type linecap = Paint.linecap = Buttcap | Squarecap | Roundcap;;

type linejoin = Paint.linejoin = Beveljoin | Roundjoin | Miterjoin;;

type linestyle = Paint.linestyle 
               = {linewidth:float;
                  linecap:linecap;
                  linejoin:linejoin;
                  dashpattern:int list};;

type fillstyle = Paint.fillstyle = Nzfill | Eofill;;
type clipstyle = Paint.clipstyle = Nzclip | Eoclip;;


let ( black : color ) = black ;;
let ( white : color ) = white ;;
let ( red : color ) = red ;;
let ( green : color ) = green ;;
let ( blue : color ) = blue ;;
let ( yellow : color ) = yellow ;;
let ( cyan : color ) = cyan ;;
let ( magenta : color ) = magenta ;;

(* END PAINT *)

(* BEGIN FRAMES *)


type frame = Frames.frame =
             {mutable xmin:float; mutable xmax:float;  
              mutable ymin:float; mutable ymax:float};;  

type extension = Frames.extension =
                 All_ext | Horiz_ext | Vertic_ext | Left_ext
               | Right_ext | Top_ext | Bottom_ext;;


let ( frame_center : frame -> point ) = frame_center ;;
let ( extend_frame : extension -> float -> frame -> frame
) = extend_frame ;;
let ( point_frame : point -> frame ) = point_frame ;;
let ( seg_frame : point list -> frame ) = seg_frame ;;
let ( ordered_angles : float * float * float -> bool
) = ordered_angles ;;
let ( arc_frame : point * float * float * float -> frame
) = arc_frame ;;
let ( curve_frame : point * point * point * point -> frame
) = curve_frame ;;
let ( merge_frames : frame list -> frame ) = merge_frames ;;
let ( compute_geom_elem_frame : geom_element -> frame
) = compute_geom_elem_frame ;;
let ( compute_frame : geom_element list -> frame
) = compute_frame ;;
let ( transform_frame : transformation -> frame -> frame
) = transform_frame ;;
let ( frame_to_frame_transform : frame -> frame -> transformation
) = frame_to_frame_transform ;;

let ( set_exact_frame_mode ) = set_exact_frame_mode;;
let ( get_exact_frame_mode ) = get_exact_frame_mode;;
let ( set_discrete_curve_number ) = set_discrete_curve_number;;
let ( get_discrete_curve_number ) = get_discrete_curve_number;;
let ( set_discrete_circle_number ) = set_discrete_circle_number;;
let ( get_discrete_circle_number ) = get_discrete_circle_number;;
let ( compute_geom_elem_convex_hull : geom_element -> point list
 ) = compute_geom_elem_convex_hull;;
let ( compute_geom_elem_list_convex_hull : geom_element list -> point list
 ) = compute_geom_elem_list_convex_hull;;
let ( merge_convex_hulls : point list list -> point list
 ) = merge_convex_hulls;;
let ( frame_of_convex_hull : point list -> frame
 ) = frame_of_convex_hull;;
let ( convex_hull_of_frame : frame -> point list
 ) = convex_hull_of_frame;;

(* END FRAMES *)

(* BEGIN SKETCHES *)


type path = Sketches.path;;
type sketch = Sketches.sketch;;


let ( recompute_sketch_hull : sketch -> sketch
 ) = recompute_sketch_hull;;
let ( sketch_frame : sketch -> frame ) = sketch_frame;;
let ( has_exact_frame_sketch : sketch -> bool
 ) = has_exact_frame_sketch;;
let ( sketch_hull : sketch -> point list ) = sketch_hull;;
let ( sketch_center : sketch -> point ) = sketch_center;;
let ( sketch_height : sketch -> float ) = sketch_height;;
let ( sketch_width : sketch -> float ) = sketch_width;;
let ( compute_size : geom_element list -> int ) = compute_size;;
let ( make_sketch : geom_element list -> sketch ) = make_sketch;;
let ( frame_sketch : frame -> sketch ) = frame_sketch;;
let ( hull_sketch : point list -> sketch ) = hull_sketch;;
let ( sketch_center : sketch -> point ) = sketch_center;;


let ( group_sketches : sketch list -> sketch ) = group_sketches ;;
let ( ungroup_sketch : sketch -> sketch list ) = ungroup_sketch ;;
let ( transform_sketch : transformation -> sketch -> sketch
) = transform_sketch ;;
let ( center_sketch : sketch -> point -> sketch ) = center_sketch;;
let ( fit_sketch_in_frame : sketch -> frame -> sketch
) = fit_sketch_in_frame ;;
let ( force_sketch_in_frame : frame -> sketch -> sketch
) = force_sketch_in_frame ;;
let ( scale_sketch : float * float -> sketch -> sketch
) = scale_sketch ;;
let ( scale_and_center_sketch : float * float -> sketch -> sketch
 ) = scale_and_center_sketch;;
let ( translate_sketch : float * float -> sketch -> sketch
) =translate_sketch ;;
let ( vflip_sketch : sketch -> sketch ) = vflip_sketch ;;
let ( hflip_sketch : sketch -> sketch ) = hflip_sketch ;;
let ( rotate_sketch : float -> sketch -> sketch) =rotate_sketch ;;
let ( besides_sketch : sketch -> sketch -> sketch
) = besides_sketch ;;
let ( over_sketch : sketch -> sketch -> sketch
) = over_sketch ;;
(*
let ( BSK) =BSK ;;
let ( OSK) =OSK ;;
*)
let ( extend_sketch_frame : extension -> float -> sketch -> sketch
) = extend_sketch_frame ;;

(* END SKETCHES *)


(* BEGIN BITMAPS *)


type bitmap = Bitmaps.bitmap;;


let ( bitmap_width : bitmap -> int ) = bitmap_width;;
let ( bitmap_height : bitmap -> int ) = bitmap_height ;;
let ( bitmap_depth : bitmap -> int ) = bitmap_depth ;;
let ( create_bitmap : int -> int -> int -> bitmap
) = create_bitmap ;;
let ( sub_bitmap : bitmap -> int * int -> int * int -> bitmap
) = sub_bitmap;;
let ( copy_bitmap : bitmap -> bitmap ) = copy_bitmap;;

let ( invert_bitmap : bitmap -> bitmap ) = invert_bitmap ;;
let ( set_pixel : bitmap -> int -> int -> int -> unit
) = set_pixel ;;
let ( get_pixel : bitmap -> int -> int -> int
) = get_pixel ;;

let ( convert_bitmap : int * (int -> int) -> bitmap -> bitmap
) = convert_bitmap ;;
let ( map_bitmap : (int -> int) -> bitmap -> bitmap 
) = map_bitmap ;;
let ( read_bitmap : int -> string -> bitmap ) = read_bitmap ;;
let ( write_bitmap : bitmap -> string -> unit ) = write_bitmap ;;
let ( bitmap_frame : bitmap -> frame ) = bitmap_frame ;;
let ( bitmap_hull : bitmap -> point list ) = bitmap_hull;;

(* END BITMAPS *)

(* BEGIN FONTS *)


type font_style = Fonts.font_style
                = Courier
                | Courier_Oblique
                | Courier_Bold
                | Courier_BoldOblique
                | Times_Roman
                | Times_Bold
                | Times_Italic  
                | Times_BoldItalic
                | Helvetica
                | Helvetica_Bold
                | Helvetica_Oblique
                | Helvetica_BoldOblique
                | Symbol
                | Other_font_style of string
;;

type font = Fonts.font
          = { font_style : font_style ;  font_size : float }
;;

type font_description = Fonts.font_description =
     { font_descr_filename : string;
       mutable font_descr_name : string ; 
       mutable font_descr_height : float ; 
       mutable font_descr_width : float ;
       mutable font_descr_descr : float vect;
       mutable font_descr_descr_bbox : ((float * float ) * (float * float)) vect }
;;




let ( add_font : font_style * font_description -> unit
) = add_font ;;
let ( remove_font : font_style * font_description -> unit
) = remove_font ;;
let ( print_font_list) =print_font_list ;;
let ( print_info_font : font_description -> unit
) = print_info_font ;;
let ( print_info_all_fonts) =print_info_all_fonts ;;

let ( default_font : unit -> font ) = default_font ;;
let ( set_default_font : font -> unit ) = set_default_font ;;
let ( reset_default_font) =reset_default_font ;;
let ( make_font : font_style -> float -> font ) = make_font;;

(* END FONTS *)

(* BEGIN TEXTS *)

let ( text_frame : font -> string -> frame ) = text_frame ;;
let ( text_width : font -> string -> float ) = text_width ;;
let ( text_height : font -> string -> float ) = text_height ;;

(* END TEXTS *)

(* BEGIN GRAPHICS_DEFAULTS *)

(*let ( default_graphic_state) =default_graphic_state ;;*)
let ( default_linewidthcoef) =default_linewidthcoef ;;
let ( default_linecap : unit -> linecap ) =default_linecap ;;
let ( default_linejoin : unit -> linejoin) =default_linejoin ;;
let ( default_dashpattern) =default_dashpattern ;;
let ( default_color : unit -> color ) =default_color ;;
let ( default_closed_sketch) =default_closed_sketch ;;
let ( default_fillstyle : unit -> fillstyle) =default_fillstyle ;;
let ( default_miterlimit) =default_miterlimit ;;
let ( default_linestyle : frame -> linestyle) =default_linestyle ;;
let ( set_default_linewidthcoef) =set_default_linewidthcoef ;;
let ( set_default_color : color -> unit) =set_default_color ;;
let ( set_default_closed_sketch) =set_default_closed_sketch ;;
let ( set_default_fillstyle : fillstyle -> unit) =set_default_fillstyle ;;
let ( set_default_linecap : linecap -> unit) =set_default_linecap ;;
let ( set_default_linejoin : linejoin -> unit) =set_default_linejoin ;;
let ( set_default_dashpattern) =set_default_dashpattern ;;
let ( set_default_miterlimit) =set_default_miterlimit ;;
(* END GRAPHICS_DEFAULTS *)



(* BEGIN PICTURES *)


type interface = Pictures.interface =
    No_handle
  | One_handle of point * point
  | Handles of (string * (point * point)) list
;;
type picture = Pictures.picture;;
type alignment = Pictures.alignment =
    Align_Right
  | Align_Left
  | Align_Center
  | Align_Top
  | Align_Bottom
;;


let ( recompute_picture_hull : picture -> picture
 ) = recompute_picture_hull;;
let ( picture_frame : picture -> frame ) =  picture_frame;;
let ( has_exact_frame_picture : picture -> bool
 ) = has_exact_frame_picture;;
let ( has_exact_frame : picture -> bool ) = has_exact_frame;;
let ( picture_hull : picture -> point list ) = picture_hull;;
let ( picture_center : picture -> point) =picture_center ;;
let ( picture_input_interface : picture -> interface
) = picture_input_interface ;;
let ( picture_output_interface : picture -> interface
) = picture_output_interface ;;
let ( picture_height : picture -> float ) =picture_height ;;
let ( picture_width : picture -> float ) =picture_width ;;
(*let ( picture_color) =picture_color;;*)
let ( change_color_picture : color -> picture -> picture
) = change_color_picture ;;
let ( change_linestyle_picture : linestyle -> picture -> picture
) = change_linestyle_picture ;;
let ( change_linewidth_picture : float -> picture -> picture
) = change_linewidth_picture ;;
let ( set_picture_interfaces : picture -> interface * interface -> picture
) = set_picture_interfaces ;;

let ( make_blank_picture : float * float -> picture
) = make_blank_picture ;;
let ( blank_rectangle : float * float -> picture)
= blank_rectangle;;
let ( blank_square : float -> picture)
= blank_square;;

let ( make_draw_picture : linestyle * color -> sketch -> picture
) =make_draw_picture ;;
let ( make_closed_draw_picture : linestyle * color -> sketch -> picture
) = make_closed_draw_picture ;;
let ( make_default_draw_picture : sketch -> picture
) =make_default_draw_picture ;;
let ( make_default_closed_draw_picture : sketch -> picture
) =make_default_closed_draw_picture ;;
let ( make_fill_picture : fillstyle * color -> sketch -> picture
) =make_fill_picture ;;
let ( make_default_fill_picture : sketch -> picture
) =make_default_fill_picture ;;
let ( clip_picture : clipstyle -> sketch -> picture -> picture
) =clip_picture ;;
let ( make_bitmap_picture : bitmap -> picture ) =make_bitmap_picture ;;
let ( make_bitmap_mask_picture : bitmap -> color -> bool -> picture
) =make_bitmap_mask_picture ;;
let ( make_default_bitmap_mask_picture : bitmap -> picture
) =make_default_bitmap_mask_picture ;;
let ( make_text_picture : font -> color -> string -> picture
) =make_text_picture ;;
let ( make_default_text_picture : string -> picture
) =make_default_text_picture ;;
let ( make_frame_picture : linestyle * color -> frame -> picture
) =make_frame_picture ;;
let ( make_default_frame_picture : frame -> picture
) =make_default_frame_picture ;;
let ( make_hull_picture : linestyle * color -> point list -> picture
 ) = make_hull_picture;;
let ( make_default_hull_picture : point list -> picture
 ) = make_default_hull_picture;;
let ( get_picture_frame_as_picture_with_lsty_and_color
 : picture -> linestyle * color -> picture
 ) = get_picture_frame_as_picture_with_lsty_and_color;;
let ( get_picture_frame_as_picture : picture ->  picture
 ) = get_picture_frame_as_picture;;
let ( get_picture_hull_as_picture_with_lsty_and_color
 :picture -> linestyle * color -> picture
 ) = get_picture_hull_as_picture_with_lsty_and_color;;
let ( get_picture_hull_as_picture :picture ->  picture
 ) = get_picture_hull_as_picture;;


let ( group_pictures : picture list -> picture ) =group_pictures ;;
let ( ungroup_picture : picture -> picture list ) =ungroup_picture ;;
let ( transform_picture : transformation -> picture -> picture
) =transform_picture ;;
let ( center_picture : picture -> point -> picture
) =center_picture ;;
let ( scale_and_center_picture : float * float -> picture -> picture
 ) = scale_and_center_picture;;
let ( translate_picture : float * float -> picture -> picture
) =translate_picture ;;
let ( translate : float * float -> picture -> picture ) =translate ;;
let ( rotate_picture : float -> picture -> picture
) =rotate_picture ;;
let ( rotate : float -> picture -> picture ) =rotate ;;
let ( scale_picture : float * float -> picture -> picture
) =scale_picture ;;
let ( scale : float * float -> picture -> picture ) =scale ;;
let ( fit_picture_in_frame : picture -> frame -> picture
) =fit_picture_in_frame ;;
let ( fit_in_frame : picture -> frame -> picture ) =fit_in_frame ;;
let ( force_picture_in_frame : frame -> picture -> picture
) =force_picture_in_frame ;;
let ( force_in_frame : frame -> picture -> picture
) =force_in_frame ;;
let ( add_frame_to_picture : picture -> picture
) =add_frame_to_picture ;;
let ( add_frame : picture -> picture ) =add_frame ;;
let ( vflip_picture : picture -> picture ) =vflip_picture ;;
let ( vflip : picture -> picture ) =vflip ;;
let ( hflip_picture : picture -> picture ) =hflip_picture ;;
let ( hflip : picture -> picture ) =hflip ;;
let ( subpicture_transformations : picture -> picture -> transformation list
) =subpicture_transformations;;
let ( subpicture_colors : picture -> picture -> color list
) =subpicture_colors;;
let ( subpicture_clips : picture -> picture -> path list
) =subpicture_clips;;
let ( subpictures : picture -> picture list ) =subpictures;;



let ( align_horizontally_picture_list : alignment -> picture list -> picture
) =align_horizontally_picture_list ;;
let ( align_horizontally : alignment -> picture list -> picture
) =align_horizontally ;;


let ( align_vertically_picture_list : alignment -> picture list -> picture
) =align_vertically_picture_list ;;
let ( align_vertically : alignment -> picture list -> picture
) =align_vertically ;;

let ( besides_picture : picture -> picture -> picture
) =besides_picture ;;
let ( over_picture : picture -> picture -> picture
) =over_picture ;;
let ( besides : picture -> picture -> picture ) =besides ;;
let ( over : picture -> picture -> picture ) =over ;;

let ( compose_horizontally_picture_list : picture list -> picture
) =compose_horizontally_picture_list ;;
let ( compose_horizontally : picture list -> picture
) =compose_horizontally ;;
let ( compose_vertically_picture_list : picture list -> picture
) =compose_vertically_picture_list ;;
let ( compose_vertically : picture list -> picture
) =compose_vertically ;;
let ( extend_picture_frame : extension -> float -> picture -> picture
) =extend_picture_frame ;;
let ( named_attach_pictures : picture * picture -> string * string -> picture
) =named_attach_pictures ;;
let ( attach_pictures : picture * picture -> picture ) =attach_pictures ;;
let ( make_textblock_picture
 : alignment -> float -> font -> color -> string list -> picture
) =make_textblock_picture ;;

let ( circletexttop : font -> string -> point -> float -> picture
) =circletexttop ;;
let ( circletextbottom : font -> string -> point -> float -> picture
) =circletextbottom ;;



(* END PICTURES *)



(* BEGIN TRPS *)


let ( set_path_size_limit) =set_path_size_limit ;;
let ( set_symbol_max) =set_symbol_max ;;
let ( set_gsave_max) =set_gsave_max ;;
let ( set_dash_size_limit) =set_dash_size_limit ;;

(* END TRPS *)


(* BEGIN DISPLAY *)
let ( page_height) =page_height ;;
let ( page_width) =page_width ;;
let ( page_hmargin) =page_hmargin ;;
let ( page_vmargin) =page_vmargin ;;
let ( set_page_height) =set_page_height ;;
let ( set_page_width) =set_page_width ;;
let ( set_page_hmargin) =set_page_hmargin ;;
let ( set_page_vmargin) =set_page_vmargin ;;
let ( compute_display_frame : picture -> frame
) =compute_display_frame ;;
let ( center_and_adjust_picture : picture -> picture
) =center_and_adjust_picture ;;
let ( center_and_adjust : picture -> picture
) =center_and_adjust ;;
let ( set_frame_extension_coef ) = set_frame_extension_coef;;
let ( ps_file : picture -> string -> unit) =ps_file ;;
let ( eps_file : picture -> string -> unit) =eps_file ;;
let ( ps : picture -> unit) =ps;;
let ( eps : picture -> unit) =eps;;


(* END DISPLAY *)




(* BEGIN OPTION *)
type optional = 
   SOption of string 
 | IOption of int 
 | FOption of float
 | BOption of bool 
 | POption
 | COption of color
 | LOption of string*float
 | DOption of int list
 | JOption of linejoin
 | CapOption of linecap
 | FillOption of fillstyle
 | FontOption of font_style;;

let string name s = name,SOption s;;
let bool name b = name,BOption b;;
let int name i = name,IOption i;;
let float name i = name,FOption i;;
let option name = name,POption;;
let color name c = name,COption c;;
let dashPattern l = "dashPattern",DOption l;;
let join j = "lineJoin",JOption j;;
let cap c = "lineCap",CapOption c;;
let fillStyle f = "fillStyle",FillOption f;;
let lineLabel name pos = "lineLabel",LOption(name,pos);;
let font f = "font",FontOption f;;


exception OptionError;;
let theString ol name default =
   try match assoc name ol with SOption b -> b | _ -> raise OptionError
   with Not_found -> default;;

let theInt ol name default =
   try match assoc name ol with IOption b -> b | _ -> raise OptionError
   with Not_found -> default;;

let theFloat ol name default =
   try match assoc name ol with FOption b -> b | _ -> raise OptionError
   with Not_found -> default;;

let theBool ol name default =
   try match assoc name ol with BOption b -> b | _ -> raise OptionError
   with Not_found -> default;;
   
let theOption ol name =
   try assoc name ol;true
   with Not_found -> false;;

let theColor ol name default =
   try match assoc name ol with COption b -> b | _ -> raise OptionError
   with Not_found -> default;;

let theDashPattern ol default =
   try match assoc "dashPattern" ol with DOption b -> b | _ -> raise OptionError
   with Not_found -> default;;

let theLineJoin ol  default =
   try match assoc "lineJoin" ol with JOption b -> b | _ -> raise OptionError
   with Not_found -> default;;

let theLineCap ol default =
   try match assoc "lineCap" ol with CapOption b -> b | _ -> raise OptionError
   with Not_found -> default;;

let theFillStyle ol default =
   try match assoc "fillStyle" ol with FillOption b -> b | _ -> raise OptionError
   with Not_found -> default;;

let theLineLabel ol =
   try match assoc "lineLabel"  ol with LOption (a,b) as c -> c | _ -> raise OptionError
   with Not_found -> raise OptionError;;

let theFont ol default =
   try match assoc "font" ol with FontOption b -> b | _ -> raise OptionError
   with Not_found -> default;;

let findOption ol name = assoc name ol;;

(* END OPTION *)

(* BEGIN MLGLATEX *)
let directory_concat_string_latex = ref "/";;

let oRotation = origin_rotation;;
let arrowSketch  = 
   make_sketch[Seg[{xc=20.0;yc=0.0};{xc=20.0;yc=20.0};{xc=40.0;yc=20.0};
    {xc=40.0;yc=30.0};{xc=50.0;yc=10.0};{xc=40.0;yc= -.10.0};
    {xc=40.0;yc=0.0};{xc=20.0;yc=0.0}]];;
let arrowPict = make_fill_picture(Nzfill,Gra 0.6) arrowSketch;;
let picDir = ref "";;


let  transOfPict p =
  let lp = subpictures p in
    subpicture_transformations (hd lp)
;;


let colorOfPict p =
  let lp = subpictures p in
    subpicture_colors (hd (tl lp))
;;

let clipOfPict p =
  let lp = subpictures p in
    subpicture_clips (hd lp)
;;

let cvw = float_of_string;;
let fEq f g = let h = f-.g in le_float (-0.00001) h & le_float h 0.00001;;
let stringTexOfFloat f = if fEq f 0. then "0" else string_of_float f;; 

let blankLatexBox = make_blank_picture (0.0,0.0);;


let colorPict wd ht dp col = make_fill_picture (Eofill,col) 
  (make_sketch 
    [Seg[{xc=0.0;yc=dp};{xc=wd;yc=dp};{xc=wd;yc=ht};
         {xc=0.0;yc=ht};{xc=0.0;yc=dp}]]);;

let colorPict wd ht dp colb colf =
 group_pictures
  [make_fill_picture (Eofill,colb)
  (make_sketch
    [Seg[{xc=0.0;yc=dp};{xc=wd;yc=dp};{xc=wd;yc=ht};
         {xc=0.0;yc=ht};{xc=0.0;yc=dp}]]);
   change_color_picture colf (make_blank_picture (0.0,0.0))];;

let opaque_white = (Gra 0.0);;


let latexPictureGen opts (name,boxNumber,wd,ht,dp) = 
   let p = (colorPict (cvw wd) (cvw ht) (-.cvw dp)
            (theColor opts "background" (Gra 1.0))
            (theColor opts "foreground" opaque_white)) in
   name,(p,boxNumber);;

let latexPicture nbwhd = latexPictureGen [] nbwhd;;

let inverseTransAngle =
 let sqr x = x*.x in
 let slopeOfLine (p1,p2) = 
  let dx = p2.xc-.p1.xc and dy = p2.yc-.p1.yc in
  let long = sqrt(sqr dx +. sqr dy) in
   if le_float 0. dx then 180.*.acos (dy/.long)/.pi
    else     360. -. 180.*.acos (dy/.long)/.pi
 in fun t -> 
  let ang = slopeOfLine (transform_point t origin,transform_point t {xc=0.;yc=1.}) in
  (* print_string("% "^string_of_float ang^" => "^string_of_float  (ang*.2.)^"\n");*)
  compose_transformation (rotation origin (ang*.2.)) t;;

let printBox  oc boxNumber box  t (c,pa) =
  let print_string s = output_string oc s in 
    let base = transform_point t origin in
    let t2 = inverseTransAngle t in
    match (t2,c) with
     {m11=1.;m12=0.;m13=_;m21=0.;m22=1.;m23=_},(Gra 1.0) ->
       print_string
        ("\\put("^stringTexOfFloat base.xc^","
         ^stringTexOfFloat base.yc^"){\\"^box^string_of_int boxNumber^"}\n")
    | _,(Gra 1.0) ->
             print_string
              ("\\mlPut{("^stringTexOfFloat base.xc^","
               ^stringTexOfFloat base.yc^")}{"^
               list_it (fun f s -> stringTexOfFloat f ^" "^s)
                  [t2.m11;t2.m21;t2.m12;t2.m22] "}{\\"^box ^
               string_of_int boxNumber^"}\n")
          | _,(Gra x) ->
             print_string
              ("\\mlPutg{("^stringTexOfFloat base.xc^","
               ^stringTexOfFloat base.yc^")}{"^
               list_it (fun f s -> stringTexOfFloat f ^" "^s)
                  [t2.m11;t2.m21;t2.m12;t2.m22] "}{"^stringTexOfFloat x^"}{\\"^box ^
               string_of_int boxNumber^"}\n")
          | _,(Rgb (r,g,b)) ->
             print_string
              ("\\mlPutrgb{("^stringTexOfFloat base.xc
               ^","^stringTexOfFloat base.yc^")}{"^
               list_it (fun f s -> stringTexOfFloat f ^" "^s)
                  [t2.m11;t2.m21;t2.m12;t2.m22] "}{");
              flush std_out;
(*              special_display_path pa " clip newpath ";*)
              print_string  (" "^ (list_it (fun f s -> stringTexOfFloat f^" "
                          ^s) [r;g;b] "}{\\"^box ^
                          string_of_int boxNumber^"}\n"))
          | _,(Hsb (h,s,b)) ->
             print_string
              ("\\mlPuthsb{("^stringTexOfFloat base.xc^","
               ^stringTexOfFloat base.yc^")}{"^
               list_it (fun f s -> stringTexOfFloat f ^" "^s)
                  [t2.m11;t2.m21;t2.m12;t2.m22] "}{"^
                  (list_it (fun f s -> stringTexOfFloat f^" "^s) 
                       [h;s;b] "}{\\"^box ^
                  string_of_int boxNumber^"}\n"))
;;

let rec printBoxList f1 f2  ls = 
  match  ls with 
    ([],[],[]) -> ()
  | ([t],[c],[p]) -> f2 t (c,p)
  | ((t::tl),(c::cl),(p::pl)) -> f1 t (c,p); printBoxList f1 f2 (tl,cl,pl)
  |  (l1,l2,l3) ->  failwith ("trans "^string_of_int (list_length l1)^" col"^
              string_of_int (list_length l2)^" clip"^
              string_of_int (list_length l3))
;;


let makeLatexPicture p' exts name =  
  let oc = open_out (!picDir^(!directory_concat_string)^name^".tex") in 
  let print_string s = output_string oc s in 
  let p = begin let f = picture_frame p' in 
                  translate(-.f.xmin,-.f.ymin) p' 
          end in
    eps_file p (!picDir^(!directory_concat_string)^name);
  let f = picture_frame p in 
    print_string ("%----------- begin of "^name^".tex -----------\n");
    print_string ("\\begin{picture}("^string_of_float (f.xmax-.f.xmin)^","
      (* modif -f.xmin a la place de -f.ymin *)
     ^string_of_float (f.ymax-.f.ymin)^")(0,0)\n");
    print_string ("\\put(0,0){\\special{psfile="^ !picDir^
      (!directory_concat_string_latex)^name^".ps}}\n");
   (*print_string ("\\put(0,0){\\makebox(0,0){$\\times$}}\n");*)
   do_list (fun (_,(e,boxNumber)) -> flush oc;
    let new_printBox = printBox oc boxNumber in    
         let l1 = transOfPict e p in
         let l2 = colorOfPict e p in 
         let l3 = clipOfPict e p in
            printBoxList (new_printBox "copy") (new_printBox "box") (l1,l2,l3))     exts;
   print_string "\\end{picture}\n";
   print_string ("%------------ end of "^name^".tex ------------\n");
   close_out oc;;

let latexBoxTable=ref ([]:(string * (picture *  int)) list );;
let add_latexBox lb = latexBoxTable:=(latexPicture lb)::!latexBoxTable;;
let empty_latexBoxTable () = latexBoxTable:=[];;
let latexBox s = 
  try fst (assoc s !latexBoxTable)
  with Not_found -> failwith ("latexBox : "^s^" latexBox unknown");;


let latexBoxGen opts s =
try 
  let p = fst(assoc s !latexBoxTable) in
  let lp = subpictures p in
  match lp with
    [fill;blank] ->
     group_pictures
       [change_color_picture (theColor opts "background" (Gra 1.0)) fill;
        change_color_picture (theColor opts "foreground" (Gra 1.0))
        (force_picture_in_frame {xmin=0.;xmax=0.;ymin=0.;ymax=0.} blank)]
   | _ -> p
with Not_found -> failwith ("latexBoxGen : "^s^" latexBox unknown")
;;

let is_latex_mode = ref false;;
let latex_mode () = !is_latex_mode;;
let set_latex_mode () = is_latex_mode:=true;;

(* END MLGLATEX *)

directory_concat_string:= "\\";;
change_graphics_directory "C:\\ocaml\\lib";;

             (********************************)
             (*Fonctions reprises de graph.ml*)
             (********************************)

type dir = Dn | Ds | De | Dw | Dne | Dnw | Dse | Dsw | Deg of float;;

let degOfDir = 
  function De -> 0. | Dne -> 45. | Dn -> 90. | Dnw -> 135.
    | Dw -> 180. | Dsw -> 225. | Ds -> 270. | Dse -> 315.
    | (Deg x) -> x;;

let sqr x = x*.x
and pAdd p1 p2 = {xc=p1.xc+.p2.xc; yc= p1.yc+.p2.yc}
and pSub p1 p2 = {xc=p1.xc-.p2.xc; yc= p1.yc-.p2.yc}
and pi = acos (-.1.)
and pMult p x = {xc=p.xc*.x;yc=p.yc*.x};;
             
let lengthOfLine(p1,p2) = 
  let dx = p2.xc-.p1.xc and dy = p2.yc-.p1.yc in sqrt(sqr dx +. sqr dy);;
  
let slopeOfLine (p1,p2) = 
  let dx = p2.xc-.p1.xc and dy = p2.yc-.p1.yc in
  let long = sqrt(sqr dx +. sqr dy) in
   if le_float 0. dy  then 180.*.acos (dx/.long)/.pi
    else     360. -. 180.*.acos (dx/.long)/.pi;;

let circlePoint r ang = {xc=r*.cosinus ang;yc=r*.sinus ang};;

(* Computes the point and the tangent angle of the x'th part of a curve  *)
let curvePos (p1,p2,p3,p4) x =
  if lt_float x 0. or gt_float x  1. then raise (Failure "curvePos");
  match int_of_float(x*.32.) with
    0 -> p1,slopeOfLine(p1,p2)
 | 32 -> p4,slopeOfLine(p3,p4)
 | i ->  
   let rec 
   findPos (p1,p2,p3,p4) (n,m) = (* m < i < m+n *)
     let p2'  = pMult(pAdd p1 p2) 0.5
     and pc   = pMult(pAdd p2 p3) 0.5
     and p3'' = pMult(pAdd p3 p4) 0.5 in
     let p3'  = pMult(pAdd p2' pc) 0.5
     and p2'' = pMult(pAdd pc p3'') 0.5 in
     let pc'  = pMult(pAdd p3' p2'') 0.5
     and j=n+m in
      if i=j then pc', slopeOfLine(p3',p2'')
      else if i<j then findPos(p1,p2',p3',pc') (n,m/2)
                  else findPos(pc',p2'',p3'',p4) (j,m/2)
   in findPos (p1,p2,p3,p4) (0,16);;


             (********************************)
             (*         Utilitaires          *)
             (********************************)

let pMilieu p q = pMult (pAdd p q) 0.5
and pProduitMixte p q = p.xc *. q.yc -. q.xc *. p.yc;;
let pProduitVectoriel p q coeff longueur =
(* (1 - coeff) * p + coeff * q + longueur * ^ (q-p)/||q-p|| *)
  let {xc = x;yc = y} = pSub q p in
  pAdd
    (pMult p (1.0-.coeff))   
    (pAdd
       (pMult q coeff)
       (pMult {xc = -.y;yc = x} (longueur/.lengthOfLine(p,q))));;

let linewidth_defaut = ref 0.1
and linecap_defaut = ref Roundcap
and linejoin_defaut = ref Roundjoin
and dashpattern_defaut = ref []
and color_defaut = ref black;;

let picture_of_geom_element
    ?(linewidth = !linewidth_defaut)
    ?(linecap = !linecap_defaut)
    ?(linejoin = !linejoin_defaut)
    ?(dashpattern = !dashpattern_defaut)
    ?(color = !color_defaut)
    g =
  make_draw_picture
    ({linewidth=linewidth;linecap=linecap;
      linejoin=linejoin;dashpattern=dashpattern},color)
    (make_sketch [g]);;  

let arc ?linewidth ?linecap ?linejoin ?dashpattern ?color
        point rayon angle1 angle2 =
  picture_of_geom_element ?linewidth ?linecap ?linejoin ?dashpattern ?color
    (Arc (point,rayon,angle1,angle2));;

let seg ?linewidth ?linecap ?linejoin ?dashpattern ?color
        liste_de_points =
  picture_of_geom_element ?linewidth ?linecap ?linejoin ?dashpattern ?color
    (Seg liste_de_points);;

let curve ?linewidth ?linecap ?linejoin ?dashpattern ?color
          p q r s =
  picture_of_geom_element ?linewidth ?linecap ?linejoin ?dashpattern ?color
    (Curve(p,q,r,s));;

let disque ?(color = !color_defaut) point rayon =
  make_fill_picture
    (Nzfill,color) (make_sketch [Arc (point,rayon,0.0,360.0)]);;


let rectangleFill ?(color = white) largeur hauteur =
  make_fill_picture
    (Nzfill,color)
    (make_sketch [Seg[origin;{xc = largeur;yc = 0.0};
         {xc = largeur;yc = hauteur};{xc = 0.0;yc = hauteur};origin]]);;

let text
    ?(style = Times_Roman)
    ?(size = 12.0)
    ?(color = !color_defaut)
    s =
  make_text_picture {font_style = style; font_size = size} color s;;

             (********************************)
             (*          Graphes             *)
             (********************************)

type type_sommet = Point | Ovale | Rectangle
and direction = G | M | D
and type_fleche = In | Out
and graphe = picture list ref * picture list ref;;

let marge_defaut = ref 2.0 (* pour sommet_ovale etc. *)

and longPointe_defaut = ref 4.0 (* pour pointes *)

and ecartArete_defaut = ref 0.0 (* pour arete *)
and ecartBoucle_defaut = ref 30.0 (* pour boucle *)
and nbrePointes_defaut = ref 0 (*0,1 ou 2 pour arete*)
and distEtiquette_defaut = ref 5.0 (* pour arete *)
and longFleche_defaut = ref 20.0
and typeSommet_defaut = ref Point

and distSommet_defaut = ref 100.;; 

let pointe (* pointe de flche *)
      ?(color = !color_defaut)
      ?(len = !longPointe_defaut)  (* longueur *)
      ?(rl = 0.75)  (* rapport largeur/longueur *)
      ?(ra1 = 0.2)  (* (distance de la base  son pt d'ancrage)/longueur *)
      ?(ra2 = 0.02) (* (distance du ct  son point d'ancrage)/longueur *)
      p             (* : point, position de la pointe *)
      {xc = x;yc = y} = (* : point, vecteur directeur *)
  let m = pAdd p (circlePoint (-. len) (180.0 *. atan2 y x /. pi))
  and l1 = rl *. 0.5 *. len 
  and l2 = ra2 *. len in
  let q = pProduitVectoriel m p 0.0 l1
  and r = pProduitVectoriel m p 0.0 (-. l1) in
  let pq = pProduitVectoriel p q 0.5 l2
  and qr = pProduitVectoriel q r 0.5 (ra1 *. len)
  and rp = pProduitVectoriel r p 0.5 l2 in
  make_fill_picture
    (Nzfill,color)
    (
    make_sketch
      [
      Curve (p,pq,pq,q);
      Curve (q,qr,qr,r);
      Curve (r,rp,rp,p)
      ]
    );;

let sommet_courant = ref (seg [origin]);;

let init_graphe () =
  sommet_courant := seg [origin];
  ((ref [],ref []):graphe);; (* sommets, arcs *)
  
let picture_of_graphe ((ls,la):graphe) = group_pictures (!la @ !ls);;

let ajouter_graphe
    sommet (* boolen *)
    ?graphe
    pict =
  (match graphe with 
    | None -> () 
    | Some ((ls,la):graphe) -> if sommet then ls := pict :: !ls 
                                else la := pict :: !la);
  pict;;

let placement_sommet
    ?graphe  (* graphe auquel ajouter le sommet *)
    ?(refSommet = !sommet_courant) (* point de rfrence *)
    ?(dirSommet = De) (* direction du point de rfrence au nouveau sommet *)
    ?(distSommet = !distSommet_defaut) (* distance du point de rf au sommet *)
    sommet = (* nouveau sommet *)
  let p = pAdd (picture_center refSommet) 
               (circlePoint distSommet (degOfDir dirSommet)) in
  let s = center_picture sommet p in
  sommet_courant := s;
  ajouter_graphe true ?graphe s;;
  
let cercle_ou_ovale
    ?graphe     (* graphe auquel ajouter le sommet *)
    ?refSommet  (* point de rfrence *)
    ?dirSommet  (* direction du point de rfrence au nouveau sommet *)
    ?distSommet (* distance du point de rf au sommet *)
    ?(linewidth = !linewidth_defaut)
    ?(dashpattern = !dashpattern_defaut)
    ?(color = !color_defaut)
    ?(colorBg = white)
    ?dims                     (* largeur,hauteur imposes si prsentes *)
    ?(marge = !marge_defaut)  (* dimensions  ajouter si dims absent *)
    cercle          (* bool *)
    p =             (* picture *)
  let (larg,haut) as dimensions = match dims with
    | None -> let l',h' = picture_width p, picture_height p in
              let l,h = if cercle then let m = max l' h' in m,m else l',h'
              and r = 2.0 *. marge and u = 1.414 in
              u *. (l +. r),u *. (h +. r)
    | Some d -> d
  and {xc = x;yc = y} as c = picture_center p in
  let sk = scale_sketch dimensions (make_sketch [Arc (c,0.5,0.0,380.0)]) in
  let ovale =
    make_draw_picture
      ({linewidth=linewidth;linecap=Roundcap;
        linejoin=Roundjoin;dashpattern=dashpattern},color) sk
  and fond =
    make_fill_picture (Nzfill,colorBg) sk in
  placement_sommet ?graphe ?refSommet ?dirSommet ?distSommet
    (group_pictures [fond;ovale;p]);;

let ovale
    ?graphe
    ?refSommet
    ?dirSommet
    ?distSommet
    ?linewidth
    ?dashpattern
    ?color
    ?colorBg
    ?dims     (* largeur,hauteur imposes si prsentes *)
    ?marge    (* dimensions  ajouter si dims absent *)
    p =       (* picture  entourer *)
  cercle_ou_ovale ?graphe ?refSommet ?dirSommet ?distSommet
      ?linewidth ?dashpattern ?color ?colorBg ?dims ?marge false p;;

let cercle
    ?graphe
    ?refSommet
    ?dirSommet
    ?distSommet
    ?linewidth
    ?dashpattern
    ?color
    ?colorBg
    ?diam           (* diamtre impos si prsent *)
    ?marge          (* dimensions  ajouter si diam absent *)
    p =             (* picture  entourer *)
  cercle_ou_ovale
    ?graphe ?refSommet ?dirSommet ?distSommet
    ?linewidth ?dashpattern ?color ?colorBg
    ?dims:(match diam with None -> None | Some d -> Some (d,d))
    ?marge true p;;

let rectangle
    ?graphe
    ?refSommet
    ?dirSommet
    ?distSommet
    ?linewidth
    ?linecap
    ?linejoin
    ?dashpattern
    ?color
    ?(colorBg = white)
    ?dims                     (* largeur,hauteur imposes si prsentes *)
    ?(marge = !marge_defaut)  (* dimensions  ajouter *)
    p =                       (* picture *)
  let (larg,haut) as dimensions = match dims with
    | None -> let l,h = picture_width p, picture_height p
              and r = 2.0 *. marge in
              l +. r,h +. r
    | Some d -> d
  and {xc = x;yc = y} as c = picture_center p in
  let larg',haut' = larg *. 0.5 , haut *. 0.5 in
  let xmax = x +. larg'
  and xmin = x -. larg'
  and ymax = y +. haut'
  and ymin = y -. haut' in
  let rectangle =
    seg ?linewidth ?linecap ?linejoin ?dashpattern ?color
      [{xc=xmax;yc=ymax};
       {xc=xmin;yc=ymax};
       {xc=xmin;yc=ymin};
       {xc=xmax;yc=ymin};
       {xc=xmax;yc=ymax}] 
  and fond = 
    center_picture (rectangleFill ~color:colorBg larg haut) c in
  placement_sommet ?graphe ?refSommet ?dirSommet ?distSommet
    (group_pictures [fond;rectangle;p]);;

let intersection_sommet type_sommet s {xc = x;yc = y} =
  let {xc = x0;yc = y0} = picture_center s
  and l = 0.5 *. picture_width s 
  and h = 0.5 *. picture_height s in
  let dx = x -. x0
  and dy = y -. y0 in
  let t = atan2 (l *. dy) (h *. dx) in
  match type_sommet with
    | Point -> {xc = x0;yc = y0}
    | Ovale -> 
        {xc = x0 +. l *. cos t; yc = y0 +. h *. sin t}
    | Rectangle -> 
       let q = truncate (4.0 *. t /. pi +. 5.0) / 2 in
       if q = 1 || q = 3 then
         let h' = if q = 1 then -. h else h in
         {xc = x0 +. h' *. dx /. dy; yc = y0 +. h'}
       else
         let l' = if q = 2 then l else -. l in
         {xc = x0 +. l'; yc = y0 +. l' *. dy /. dx};;

let etiquetteSurArete
    aligne (*boolen indiquant si les pi sont aligns*)
    etiquette
   ?dirEtiquette
    distEtiquette
    posEtiquette
    (p1,p2,p3,p4) =
  let dir = match dirEtiquette with
    | None -> 
        if (aligne && p1.xc <= p4.xc) ||
           ((not aligne) && pProduitMixte (pSub p4 p1) (pSub p2 p1) > 0.0) 
          then G else D
    | Some d -> d
  and p,angle = 
    if aligne then
      pProduitVectoriel p1 p4 posEtiquette 0.0,slopeOfLine (p1,p4)
    else
      curvePos (p1,p2,p3,p4) posEtiquette
  and {xmin=xmin;xmax=xmax;ymin=ymin;ymax=ymax} as f = picture_frame etiquette in
  if dir = M then
    group_pictures [
      center_picture (
        rectangleFill (
          1.1*.picture_width etiquette ) (1.1*.picture_height etiquette)) p;
      center_picture etiquette p]
  else
    let a = mod_float (if dir = G then angle else angle +. 180.0) 360.0
    and l,h = 0.5*.picture_width etiquette,0.5*.picture_height etiquette in
    let q = pAdd p {xc= -.distEtiquette*.sinus a;yc=distEtiquette*.cosinus a}
    and vx,vy =
      if a < 0.0 || a > 360. then failwith "etiquetteSurArete" 
      else if a < 1.0 || a > 359.0 then 0.0,h
      else if a <= 89.0 then -.l,h
      else if a < 91.0 then -.l,0.0
      else if a <= 179.0  then -.l,-.h
      else if a < 181.0  then 0.0,-.h
      else if a <= 269.0  then l,-.h
      else if a < 271.0  then l,0.0
      else  l,h in
    center_picture etiquette (pAdd q {xc = vx; yc = vy});;
  
let arete
    ?graphe
    ?linewidth
    ?dashpattern
    ?color
    ?(coeff1 = 0.5) (* position du premier point de controle *)
    ?(ecart1 = !ecartArete_defaut) (* distance du point de controle  la ligne qui joint les centres des deux sommets *)
    ?(coeff2 = 0.5)
    ?(ecart2 = !ecartArete_defaut)
    ?(nbrePointes = !nbrePointes_defaut) (*0,1 ou 2*)
    ?(longPointe = !longPointe_defaut)
    ?etiquette (* picture *)
    ?dirEtiquette (*G,M ou D;si absent,  l'exterieur si courbe et dessus sinon*)
    ?(distEtiquette = !distEtiquette_defaut)
    ?(posEtiquette = 0.5)
    ?(typeSommet1 = !typeSommet_defaut)
    ?(typeSommet2 = !typeSommet_defaut)
    sommet1
    sommet2 =
  let centre1 = picture_center sommet1
  and centre2 = picture_center sommet2 in
  let c1 = pProduitVectoriel centre1 centre2 coeff1 ecart1
  and c2 = pProduitVectoriel centre1 centre2 coeff2 ecart2 in
  let p1 = intersection_sommet typeSommet1 sommet1 c1
  and p2 = intersection_sommet typeSommet2 sommet2 c2 in
  let l = ref [] in
  (match etiquette with
    | None -> ()
    | Some e -> 
       let a = ecart1 = 0.0 && ecart2 = 0.0 in
       l := (etiquetteSurArete 
               a e ?dirEtiquette distEtiquette posEtiquette (p1,c1,c2,p2))
         :: !l);
  l := (curve ?linewidth ?dashpattern ?color p1 c1 c2 p2) :: !l;
  if nbrePointes <> 0 then (
     l := (pointe ?color ~len:longPointe p2 (pSub p2 c2)) :: !l;
     if nbrePointes = 2 then
       l := (pointe ?color ~len:longPointe p1 (pSub p1 c1)) :: !l);
  ajouter_graphe false ?graphe (group_pictures !l);;

let boucle
    ?graphe
    ?linewidth
    ?dashpattern
    ?color
    ?(angle = 70.0) (* angle d'ouverture *)
    ?(ecart = !ecartBoucle_defaut) (* distance du premier point au premier point de controle *)
    ?(longPointe = !longPointe_defaut)
    ?etiquette (* picture *)
    ?(distEtiquette = !distEtiquette_defaut)
    ?(posEtiquette = 0.5)
    ?(typeSommet = Ovale)
    dir (* :dir (Dn,Dne,...) *)
    sommet =
  let centre = picture_center sommet
  and direction = degOfDir dir in
  let a1 = direction -. angle /. 2.0
  and a2 = direction +. angle /. 2.0 in
  let v1 = circlePoint 1.0 a1
  and v2 = circlePoint 1.0 a2 in
  let p1 = intersection_sommet typeSommet sommet (pAdd centre v1)
  and p2 = intersection_sommet typeSommet sommet (pAdd centre v2) in
  let c1 = pAdd p1 (pMult v1 ecart)
  and c2 = pAdd p2 (pMult v2 ecart) in
  let l = ref [] in
  (match etiquette with
    | None -> ()
    | Some e -> 
       l := (etiquetteSurArete 
               false e distEtiquette posEtiquette (p1,c1,c2,p2)) :: !l);
  l := (curve ?linewidth ?dashpattern ?color p1 c1 c2 p2) :: !l;
  l := (pointe ?color ~len:longPointe p2 (pSub p2 c2)) :: !l;
  ajouter_graphe false ?graphe (group_pictures !l);;

let fleche
    typeFleche  (*In ou Out*)
    ?graphe
    ?linewidth
    ?dashpattern
    ?color
    ?(longFleche = !longFleche_defaut)
    ?(longPointe = !longPointe_defaut)
    ?(typeSommet = !typeSommet_defaut)
    dir (* :dir (Dn,Dne,...) *)
    sommet =
  let centre = picture_center sommet
  and direction = degOfDir dir in
  let v = circlePoint 1.0 direction in
  let p = intersection_sommet typeSommet sommet (pAdd centre v) in
  let q = pAdd p (pMult v longFleche) in
  ajouter_graphe false ?graphe (
    group_pictures [
      seg ?linewidth ?dashpattern ?color [p;q];
      let pt,vect = match typeFleche with
        | In -> p,pSub origin v | Out -> q,v in
      pointe ?color ~len:longPointe pt vect]);;

             (********************************)
             (*            Arbres            *)
             (********************************)

type 'a arbre = {etiquette : 'a ; branches : 'a foret}
and 'a foret = 'a arbre list;;

type 'a arbrebinaire =
   Nil | Noeud of 'a arbrebinaire * 'a * 'a arbrebinaire;;

let ecartH_defaut = ref 30.0
and ecartV_defaut = ref 30.0
and orient_defaut = ref Ds
and fNoeud_defaut = (fun _ -> disque origin 2.0)
and fArete_defaut =
  (fun _ _ p q -> seg [picture_center p;picture_center q]);;

let rec calcule_franges l1 l2 a delta l l' l'' l''' = match (l,l',l'',l''') with
   (ag1 :: lg1),(ad1 :: ld1),(ag2 :: lg2),(ad2 :: ld2) -> 
      let a', delta' =
         if a +. ag2 -. ad1 < 1.
            then 1. , delta +. 1. -. a -. ag2 +. ad1
            else a +. ag2 -. ad1 , delta in
      let lg1',ld2',delta'' =
        calcule_franges
          (ad1 +. l1 -. ag1) (ad2 +. l2 -. ag2) a' delta'  lg1 ld1 lg2 ld2 in
      ag1 :: lg1',ad2 :: ld2',delta'' |
   lg1,(ad1 :: ld1),[],[] -> lg1 , ad1 -. a -. l2 :: ld1 , delta |
   [],[],(ag2 :: lg2),ld2 -> ag2 +. l1 +. a :: lg2 , ld2 , delta |
   _ -> [] , [] , delta;;

let ecarts_arbre,            (* 'a arbre -> ('a * float) arbre *)
    ecarts_foret' =          (* 'a foret -> ('a * float) foret *)
   let largeur f = List.fold_right ~f:(fun {etiquette=(_,d)} s -> d +. s) f ~init:0. in
   let rec aa {etiquette=x;branches=f} =
        match f with
           [] -> {etiquette=(x,0.);branches=[]} , ([],[]) |
           _ -> let f',(lg,ld) = ff f in
                let e = largeur f' /. 2. in
                let rec transf e = function
                   [] -> [] |
                   {etiquette=(x,d);branches=f} :: l ->
                      {etiquette=(x,e);branches=f} :: (transf (e +. d) l) in
                {etiquette=(x,0.);branches=transf (-. e) f'} ,
                                                     ((-. e) :: lg , e :: ld)
   and ff = function
        [] -> failwith "ff" |
        [a] -> let a', c = aa a in
               [a'] , c |
        a :: f ->
           let {etiquette=(x,_);branches=f1},(lg1,ld1) = aa a
           and f',(lg2,ld2) = ff f in
           let lg,ld,delta =
              calcule_franges 0. (largeur f') 1. 1. lg1 ld1 lg2 ld2 in
           {etiquette=(x,delta);branches=f1} :: f' , (lg,ld) in
   (function a -> fst(aa a)) , (function f -> fst(ff f));;

let ecarts_foret f = 
  let rec fct aux = function
    | [] -> []
    | {etiquette = (x,e);branches = b} :: f -> 
        {etiquette = (x,aux);branches = b} :: fct (aux +. e) f in
  fct 0.0 (ecarts_foret' f);;


let ecarts_arbrebinaire a = (*'a arbrebinaire -> ('a * float) arbrebinaire*)
   let rec ab = function
      Nil -> Nil , ([],[]) |
      Noeud(Nil,x,Nil) -> Noeud(Nil,(x,0.),Nil) , ([],[]) |
      Noeud(u,x,Nil) ->
         let u', (lg,ld) = ab u in
         Noeud(u',(x,0.5),Nil) , (-.0.5 :: lg,-.0.5 :: ld) |
      Noeud(Nil,x,v) ->
         let v', (lg,ld) = ab v in
         Noeud(Nil,(x,0.5),v') , (0.5 :: lg,0.5 :: ld) |
      Noeud(u,x,v) ->
         let u' , (lg1,ld1) = ab u
         and v' , (lg2,ld2) = ab v in
         let lg,ld,d = calcule_franges 0. 0. 1. 1. lg1 ld1 lg2 ld2 in
         Noeud(u',(x,d /. 2.),v'),
         (-. d /. 2. :: lg,d /. 2. :: ld) in
   fst(ab a);;

let positions_arbre,positions_foret,positions_arbrebinaire =
  let trpl 
      ?(orient = !orient_defaut)
      ?(ecartH = !ecartH_defaut)
      ?(ecartV = !ecartV_defaut)
      ?(fNoeud = fNoeud_defaut) () =
    let point pt e =
        pAdd 
          pt
          (match orient with
             | Ds -> {xc = e *. ecartH;yc = -. ecartV}
             | Dn -> {xc = e *. ecartH;yc = ecartV}
             | De -> {xc = ecartV;yc = -. e *. ecartH}
             | Dw -> {xc = -. ecartV;yc = -. e *. ecartH}
             | _ -> failwith "orientation incorrecte") in
    let rec pa pt {etiquette = (x,e);branches = l} =
      let pt' = point pt e in
      {etiquette = (x,center_picture (fNoeud x) pt');branches = pf pt' l}
    and pf pt = List.map ~f:(pa pt)
    and pab pt = function
      | Nil -> Nil
      | Noeud (u,(x,e),v) ->
          let pt' = point pt e in
          Noeud (pab (point pt (-.e)) u,
                 (x,center_picture (fNoeud x) pt),
                 pab (point pt e) v) in
    pa,pf,pab in
  (
  fun ?orient ?ecartH ?ecartV ?fNoeud a ->
    let pa,_,_ = trpl ?orient ?ecartH ?ecartV ?fNoeud () in
    pa origin (ecarts_arbre a)
  ),
  (
  fun ?orient ?ecartH ?ecartV ?fNoeud a ->
    let _,pf,_ = trpl ?orient ?ecartH ?ecartV ?fNoeud () in
    pf origin (ecarts_foret a)
  ),
  (
  fun ?orient ?ecartH ?ecartV ?fNoeud a ->
    let _,_,pab = trpl ?orient ?ecartH ?ecartV ?fNoeud () in
    pab origin (ecarts_arbrebinaire a)
  );;

let listes_a ?(fArete  = fArete_defaut) ln la =
  let rec listes {etiquette = (x,p);branches = l} =
    ln := p :: !ln;
    List.iter
        ~f:(
          function {etiquette = (y,q)} as b -> 
            la := (fArete x y p q) :: !la;
            listes b)
        l in
    listes;;

let arbres
    ?orient
    ?ecartH
    ?ecartV
    ?fNoeud
    ?fArete
    a =
  let a = positions_arbre ?orient ?ecartH ?ecartV ?fNoeud a
  and ln = ref []
  and la = ref [] in
  listes_a ?fArete ln la a;
  group_pictures (!la @ !ln),a;;

let arbre ?orient ?ecartH ?ecartV ?fNoeud ?fArete a =
  fst(arbres ?orient ?ecartH ?ecartV ?fNoeud ?fArete a);;

let forets
    ?orient
    ?ecartH
    ?ecartV
    ?fNoeud
    ?fArete
    f =
  let f = positions_foret ?orient ?ecartH ?ecartV ?fNoeud f
  and ln = ref []
  and la = ref [] in
  List.iter ~f:(listes_a ?fArete ln la) f;
  group_pictures (!la @ !ln),f;;

let foret ?orient ?ecartH ?ecartV ?fNoeud ?fArete a =
  fst(forets ?orient ?ecartH ?ecartV ?fNoeud ?fArete a);;

let arbrebinaires
    ?orient
    ?ecartH
    ?ecartV
    ?fNoeud
    ?(fArete = fArete_defaut)
    a =
  let a = positions_arbrebinaire ?orient ?ecartH ?ecartV ?fNoeud a
  and ln = ref []
  and la = ref [] in
  let rec listes = function
    | Nil -> ()
    | Noeud (u,(x,p),v) ->
       (
        ln := p :: !ln;
        (match u with 
           | Nil -> ()
           | Noeud(_,(y,q),_) ->
              (la := (fArete x y p q) :: !la; listes u)
        );
        (match v with 
           | Nil -> ()
           | Noeud(_,(y,q),_) ->
              (la := (fArete x y p q) :: !la; listes v)
        )
       ) in
  listes a;
  group_pictures (!la @ !ln),a;;

let arbrebinaire ?orient ?ecartH ?ecartV ?fNoeud ?fArete a =
  fst(arbrebinaires ?orient ?ecartH ?ecartV ?fNoeud ?fArete a);;

let arbre_of_string,foret_of_string =
   let couple s =
      let s = s ^ "#"
      and k = ref 0 in
      let nth() = s.[!k] in
      let rec arbre() =
         let l = ref [] in
         while nth() <> '(' do
            l := nth() :: !l;
            incr k
         done;
         let a = String.create (List.length !l) in
         let _ =
           List.fold_left 
             ~f:(fun i c -> a.[i] <- c; i + 1) 
             ~init:0 
             (List.rev !l) in
         {etiquette = a;branches = foret()}
      and foret() =
         incr k;
         let l = ref [] in
         while nth() <> ')' do
            l := arbre() :: !l
         done;
         incr k;
         List.rev !l in
      arbre,foret in
   (function s -> fst (couple s) ()),(function s -> snd (couple s) ());;

let arbrebinaire_of_string s = 
  let rec arbrebinaire_of_arbre {etiquette = e; branches = f} =
     if e = "" then Nil
       else
     match f with
        | [] -> Noeud(Nil,e,Nil)
        | [a;b] -> Noeud(arbrebinaire_of_arbre a,e,arbrebinaire_of_arbre b)
        | _ -> failwith "arbrebinaire_of_arbre" in
  arbrebinaire_of_arbre(arbre_of_string s);;

             (********************************)
             (*        Bitmaps pgm           *)
             (********************************)

let force_input_byte canal_in =
  try input_byte canal_in 
  with End_of_file -> seek_in canal_in (pos_in canal_in + 1);26;;

let read_pgm fichier_pgm =
    let erreur () = failwith "Incapable de lire fichier PGM" in
    let canal_in = open_in fichier_pgm in
    let p5 = input_line canal_in in
    if p5 <> "P5" then erreur ();
    let ligne = ref "" in
    while 
        ligne := input_line canal_in;
        !ligne = "" || !ligne.[0] = '#' do 
      ()
    done;
    let largeur,hauteur =
      let i = String.index !ligne ' ' in
      int_of_string 
        (String.sub !ligne ~pos:0 ~len:i),
      int_of_string 
        (String.sub !ligne ~pos:(i+1) ~len:(String.length !ligne-i-1)) in
    Printf.printf "largeur = %u, hauteur = %u\n" largeur hauteur;
    let b = create_bitmap largeur hauteur 8 in
    if input_line canal_in <> "255" then erreur ();
    for i = hauteur - 1 downto 0 do for j = 0 to largeur - 1 do
      set_pixel b j i (force_input_byte canal_in)
    done; done;
    close_in canal_in;
    b;;

let write_pgm b fichier_pgm =
  let largeur = bitmap_width b
  and hauteur = bitmap_height b
  and canal_out = open_out_bin fichier_pgm in
  Printf.printf "largeur = %u, hauteur = %u\n" largeur hauteur;
  Printf.fprintf canal_out 
    "P5\n# Cr par MlGraph\n%u %u\n255\n" largeur hauteur;
  for i = hauteur - 1 downto 0 do for j = 0 to largeur - 1 do
    output_byte canal_out (get_pixel b j i)
  done; done;
  close_out canal_out;;

(************************************************************************
Autres fonctions disponibles (en commentaire car pas vraiment utiles).
Les fichiers bmg sont les fichiers bitmap de profondeur 8 de MlGraph.
Le traitement des bitmaps windows est inspir d'un programme
de Monasse: bitmaps <-> graphiques de Graphics

       bmp2bmg : string -> unit
       bmp2bmg "nom_fichier"
       cre "nom_fichier.bmg"  partir de "nom_fichier.bmp"
       (bitmap windows 256 couleurs)

       pgm2bmg : string -> unit

       read_bmp : string -> bitmap
       read_bmp "nom_fichier.bmp"
************************************************************************)
(*
let bmp2bmg fichier_bmp =
    let canal_in = open_in  (fichier_bmp ^ ".bmp")
    and canal_out = open_out (fichier_bmp ^ ".bmg")
    in
    let lit_deux_octets dcalage =
        seek_in canal_in dcalage ;
        let poids_faible = force_input_byte canal_in in
        let poids_fort = force_input_byte canal_in in
        poids_faible + 256 * poids_fort in
    let largeur = 4 * ((lit_deux_octets(18) + 3)/4) in
    let hauteur = lit_deux_octets(22) in
    Printf.printf "largeur = %u, hauteur = %u\n" largeur hauteur;
    let palette = Array.make 256 "ff" in
    let lit_couleur () =
        let b = force_input_byte canal_in in
        let v = force_input_byte canal_in in
        let r = force_input_byte canal_in in
        let _ = force_input_byte canal_in in
        let g = (30 * r + 59 * v + 11 * b) / 100 in
        let t = function
          10 -> "a" | 11 -> "b" | 12 -> "c" | 13 -> "d" | 14 -> "e" | 15 -> "f"
          | i -> string_of_int i in
        t (g / 16) ^ t (g mod 16) in
    seek_in canal_in 54 ;
    for i = 0 to 255 do
      palette.(i) <- lit_couleur()
    done ;
    for i = 1 to hauteur do
        seek_in canal_in (1078 + largeur * (hauteur-i)) ;
        for j = 1 to largeur do
          output_string canal_out 
            palette.(force_input_byte canal_in)
        done;
        if i <> hauteur then output_string canal_out "\n"
    done ;
    close_out canal_out;
    close_in canal_in ;;

let pgm2bmg fichier_pgm =
    let erreur () = failwith "Incapable de lire fichier PGM" in
    let canal_in = open_in  (fichier_pgm ^ ".pgm")
    and canal_out = open_out (fichier_pgm ^ ".bmg") in
    let p5 = input_line canal_in in
    if p5 <> "P5" then erreur ();
    let ligne = ref "" in
    while 
        ligne := input_line canal_in;
        !ligne = "" || !ligne.[0] = '#' do 
      ()
    done;
    let largeur,hauteur =
      let i = String.index !ligne ' ' in
      int_of_string 
        (String.sub !ligne ~pos:0 ~len:i),
      int_of_string 
        (String.sub !ligne ~pos:(i+1) ~len:(String.length !ligne-i-1)) in
    Printf.printf "largeur = %u, hauteur = %u\n" largeur hauteur;
    if input_line canal_in <> "255" then erreur ();
    let conversion g =
        let t = function
          10 -> "a" | 11 -> "b" | 12 -> "c" | 13 -> "d" | 14 -> "e" | 15 -> "f"
          | i -> if i < 10 then string_of_int i else erreur () in
        t (g / 16) ^ t (g mod 16) in
    for i = 1 to hauteur do
        for j = 1 to largeur do
            output_string canal_out(conversion (force_input_byte canal_in))
        done;
        if i <> hauteur then output_string canal_out "\n"
    done ;
    close_out canal_out ; 
    close_in canal_in ;;

let read_bmp fichier_bmp =
    let canal_in = open_in  fichier_bmp in
    let lit_deux_octets dcalage =
      seek_in canal_in dcalage;
      let poids_faible = force_input_byte canal_in in
      let poids_fort = force_input_byte canal_in in
      poids_faible + 256 * poids_fort in
    let largeur = 4 * ((lit_deux_octets 18 + 3)/4) in
    let hauteur = lit_deux_octets 22 in
    Printf.printf "largeur = %u, hauteur = %u\n" largeur hauteur;
    let palette = Array.make 256 0 in
    let lit_couleur () = (*formule utilise pour les tls noir et blanc*)
        let b = force_input_byte canal_in in
        let v = force_input_byte canal_in in
        let r = force_input_byte canal_in in
        let _ = force_input_byte canal_in in
        (30 * r + 59 * v + 11 * b) / 100 in
    seek_in canal_in 54 ;
    for i = 0 to 255 do
      palette.(i) <- lit_couleur()
    done ;
    let b = create_bitmap largeur hauteur 8 in
    for i = 0 to hauteur - 1 do for j = 0 to largeur - 1 do
          set_pixel b j i palette.(force_input_byte canal_in)
    done done ;
    close_in canal_in;
    b;;
*)
             (********************************)
             (*            mlPic             *)
             (********************************)

let mlPicMode = ref true
and mlPicConstruction = ref false
and mlPicCompteur = ref 0
and mlPicFichier = ref stdout
and mlPicNomFichier = ref "" (* sans extension => fichier.ml et fichier.tex *)
and mlPicExpression = ref "";;

let begin_mlPic fichier expression =
  mlPicCompteur := 0;
  if !mlPicMode && not (latex_mode ()) then (
    mlPicConstruction := true;
    mlPicNomFichier := fichier;
    mlPicExpression := expression;
    mlPicFichier := open_out (fichier ^ ".tex");
    output_string !mlPicFichier 
      ("% Fichier cr par " ^ fichier ^ ".ml\n\\begin{mlPic}\n")
  );;

let end_mlPic pict =
  if not (latex_mode ()) then (
    if !mlPicMode then (
      mlPicConstruction := false;
      output_string !mlPicFichier
        (" \\mlDir{use \"" ^ !mlPicNomFichier ^ ".ml\"}\n \\mlBody{" ^ 
         !mlPicNomFichier ^ "}{" ^ !mlPicExpression ^ "}\n\\end{mlPic}");
      close_out !mlPicFichier
    );
    eps_file (center_picture pict {xc=596./.2.;yc=846./.2.}) !mlPicNomFichier
  );;  

let latexText
    ?(color = !color_defaut)
    ?(colorBg = white)
    ?strMl
    s =
  incr mlPicCompteur;
  let s' = string_of_int !mlPicCompteur in
  if latex_mode () then
    latexBox s'
  else
    let string_of_color = 
      function
        | Rgb (x,y,z) -> Printf.sprintf "Rgb (%f,%f,%f)" x y z
        | Hsb (x,y,z) -> Printf.sprintf "Hsb (%f,%f,%f)" x y z
        | Gra x -> Printf.sprintf "Gra %f" x in
    if !mlPicConstruction && !mlPicMode then
      output_string !mlPicFichier
        (if color = black && colorBg = white then
           (" \\mlDef{" ^ s' ^ "}{" ^ s ^ "}\n")
         else
           (" \\mlDefGen{" ^ s' ^ "}{[color \"foreground\" (" ^
            string_of_color color ^ ");color \"background\" (" ^
            string_of_color colorBg ^ ")]}{" ^ s ^ "}\n"));
    text ~color:color (match strMl with None -> s | Some c -> c);;

type type_brace = Accolade | Parenthese | Crochet | Ligne
and pos = Pg | Pd | Ph | Pb | Pgd | Pdg | Phb | Pbh;;

let brace 
    ?(pos=Pg) 
    ?(distPlus = 0.0) 
    ?(longPlus = 0.0) 
    ?(typeBrace = Accolade)
    ?color 
    ?colorBg
    pict =
  let distance = distPlus +. 5.0
  and long = longPlus +. 4.0 in
  let vert = pos = Pg || pos = Pd || pos = Pgd || pos = Pdg in
  let l = (if vert then picture_height else picture_width) pict in
  let l' = l +. long in
  let lt =
    let lt' =
      let ta = match typeBrace with
        | Accolade -> "\\{"
        | Parenthese -> "("
        | Crochet -> "["
        | Ligne -> "|" in
      latexText ?color ?colorBg
           ("$\\left" ^ ta ^ "\\rule{0pt}{" ^
           (string_of_float l') ^
           "pt}\\right.$") in
      if latex_mode () then
        lt'
      else
        let p = origin 
        and q = {xc = 0.0; yc = l'}
        and b = {xc = 2.0; yc = 0.0} in
        seg ?color [pAdd p b;p;q;pAdd q b] in
  let r = l' /. (picture_height lt) in
  let accol =
    let v,t = 
      let h = (if vert then picture_width else picture_height) pict /. 2.0
                 +. distance in
      match pos with
       | Pg | Pgd | Pdg -> {xc = -. h; yc = 0.0},0.0
       | Pd -> {xc = h; yc = 0.0},180.0
       | Ph | Phb | Pbh -> {xc = 0.0; yc = h},-. 90.0
       | Pb -> {xc = 0.0; yc = -. h},90.0 in
    scale (r,r)
      (rotate t
         (center_picture lt (pAdd (picture_center pict) v))) in
  let accol' =
    transform_picture (rotation (picture_center pict) 180.0) accol in 
  group_pictures 
    ([pict;accol]@(match pos with Pgd|Pdg|Phb|Pbh -> [accol'] | _ -> []));;

